package com.asreader.asfinger;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.Context;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.ListView;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.TextView;
import android.widget.Toast;

import androidx.appcompat.app.AppCompatActivity;

import com.asreader.asfingersdk.AsFingerSDK;
import com.asreader.asfingersdk.event.AsFingerEventListener;
import com.asreader.asfingersdk.type.AsFingerConnectionType;
import com.asreader.asfingersdk.type.DeviceType;

public class SearchAcitvity extends AppCompatActivity implements View.OnClickListener, RadioGroup.OnCheckedChangeListener {


    private ImageButton mIbBack;
    private RadioButton mRbSpp;
    private RadioButton mRbBle;
    private RadioGroup mRbBlueMode;
    private BluetoothAdapter mBtAdapter;
    private ArrayAdapter<String> mNewDevicesAdapter;
    private AsFingerSDK mAsFingerSDK;
    private EditText mEtConnectedDevice;
    private Button mBtConnectDevice;
    private static DeviceType deviceType = DeviceType.spp;
    private Button mStartDiscovery;
    private boolean isFinishedDiscovery = true;
    private TextView mTitleNewDevices;
    private BluetoothDevice mBluetoothDevice;
    private String mConnectedDeviceName = "";

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_search_acitvity);
        mBtAdapter = BluetoothAdapter.getDefaultAdapter();

        initView();

        mAsFingerSDK = AsFingerSDK.getInstance();

    }


    private void initView() {
        mRbSpp = (RadioButton) findViewById(R.id.rb_spp);
        mRbSpp.setChecked(true);
        mRbBle = (RadioButton) findViewById(R.id.rb_ble);
        mRbBlueMode = (RadioGroup) findViewById(R.id.rb_bluemode);
        mRbBlueMode.setOnCheckedChangeListener(this);
        mIbBack = (ImageButton) findViewById(R.id.ib_back);
        mIbBack = (ImageButton) findViewById(R.id.ib_back);
        mBtConnectDevice = (Button) findViewById(R.id.bt_connect_device);
        mEtConnectedDevice = (EditText) findViewById(R.id.et_connected_device);
        mStartDiscovery = (Button) findViewById(R.id.start_discovery);
        mTitleNewDevices = (TextView) findViewById(R.id.title_new_devices);
        mStartDiscovery.setOnClickListener(this);
        mIbBack.setOnClickListener(this);
        mBtConnectDevice.setOnClickListener(this);
        mNewDevicesAdapter = new ArrayAdapter<String>(this,
                R.layout.device_name);
        // Find and set up the ListView for newly discovered devices
        ListView newDevicesRecyclerView = (ListView) findViewById(R.id.new_devices);
        newDevicesRecyclerView.setAdapter(mNewDevicesAdapter);
        newDevicesRecyclerView.setOnItemClickListener(mDeviceClickListener);

    }

    @Override
    protected void onStart() {
        super.onStart();
        if (mBtAdapter == null) {
            return;
        }
        if (!mBtAdapter.isEnabled()) mBtAdapter.enable();
        mAsFingerSDK.setActivity(this);
        mEtConnectedDevice.setText(mConnectedDeviceName);
        if (deviceType == DeviceType.ble) {
            mRbBle.setChecked(true);
        } else if (deviceType == DeviceType.spp) {
            mRbSpp.setChecked(true);
        } else if (deviceType == DeviceType.unknow) {
            mRbSpp.setChecked(false);
            mRbBle.setChecked(false);
        }
        if (mAsFingerSDK.connectedDevice() == null) {
            mBtConnectDevice.setText(getResources().getString(R.string.connect_device));
            return;
        } else {
            mBtConnectDevice.setText(getResources().getString(R.string.disconnect_device));
            mBluetoothDevice = mAsFingerSDK.connectedDevice();
            mConnectedDeviceName = mBluetoothDevice.getName();
            mEtConnectedDevice.setText(mConnectedDeviceName);
            if (mAsFingerSDK.deviceType() == DeviceType.ble) {
                mRbBle.setChecked(true);
            } else if (mAsFingerSDK.deviceType() == DeviceType.spp) {
                mRbSpp.setChecked(true);
            }
            mRbSpp.setEnabled(false);
            mRbBle.setEnabled(false);

        }


    }

    @Override
    protected void onResume() {
        super.onResume();
        mAsFingerSDK.setAsFingerEventListener(mAsFingerEventListener);
    }

    @Override
    protected void onPause() {
        super.onPause();
        mAsFingerSDK.stopDiscovery();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        mAsFingerEventListener = null;
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.bt_connect_device:
                if (mAsFingerSDK.connectedDevice() != null) {
                    mAsFingerSDK.disconnect();
                } else {
                    if (mBluetoothDevice != null) {
                        mAsFingerSDK.connect(mBluetoothDevice, deviceType);
                    }

                }
                break;
            case R.id.start_discovery:
                if (isFinishedDiscovery) {
                    mNewDevicesAdapter.clear();
                    mAsFingerSDK.startDiscovery();
                    mTitleNewDevices.setText(R.string.discoverying);
                    mStartDiscovery.setText(R.string.stop_discovery);
                    isFinishedDiscovery = false;
                } else {
                    mAsFingerSDK.stopDiscovery();
                }
                break;
            case R.id.ib_back:
                finish();
                break;
        }
    }


    @Override
    public void onCheckedChanged(RadioGroup group, int checkedId) {
        switch (checkedId) {
            case R.id.rb_spp:
                mRbSpp.setChecked(true);
                deviceType = DeviceType.spp;
                break;
            case R.id.rb_ble:
                mRbBle.setChecked(true);
                deviceType = DeviceType.ble;
                break;
        }

    }


    private AdapterView.OnItemClickListener
            mDeviceClickListener = new AdapterView.OnItemClickListener() {
        public void onItemClick(AdapterView<?> av, View v, int arg2, long arg3) {
            String info = ((TextView) v).getText().toString();
            if (info.equals(getResources().getText(R.string.none_paired).toString())) {
                return;
            } else {
                String address = info.substring(info.length() - 17);
                mBluetoothDevice = mBtAdapter.getRemoteDevice(address);
                mAsFingerSDK.connect(mBluetoothDevice, deviceType);
            }

        }
    };


    private AsFingerEventListener mAsFingerEventListener = new AsFingerEventListener() {

        @Override
        public void onStateChanged(AsFingerConnectionType connectionType) {
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    switch (connectionType) {
                        case connected:
                            mConnectedDeviceName = mAsFingerSDK.connectedDevice().getName();
                            mEtConnectedDevice.setText(mConnectedDeviceName);
                            mBtConnectDevice.setText(getResources().getString(R.string.disconnect_device));
                            bluetoothMode(false);
                            saveBluetoothDevcie();
                            Toast.makeText(SearchAcitvity.this,
                                    getResources().getString(R.string.toast_connect_device) + " " + mAsFingerSDK.connectedDevice().getName(), Toast.LENGTH_LONG).show();
                            break;
                        case disconnected:
                            mBtConnectDevice.setText(getResources().getString(R.string.connect_device));
                            bluetoothMode(true);
                            Toast.makeText(SearchAcitvity.this,
                                    getResources().getString(R.string.toast_disconnect_device), Toast.LENGTH_LONG).show();
                            break;
                        case connecting:
                            Toast.makeText(SearchAcitvity.this,
                                    getResources().getString(R.string.toast_connecting_device), Toast.LENGTH_LONG).show();
                            break;
                    }
                }
            });

        }

        @Override
        public void receivedBarcodeData(byte[] barcodeData) {

        }

        @Override
        public void receivedBattery(int battery) {


        }

        @Override
        public void receivedFirmwareVersion(String firmwareVersion) {

        }

        @Override
        public void receivedData(byte[] data) {

        }

        @Override
        public void receivedDevice(BluetoothDevice device) {
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    if (mNewDevicesAdapter.getPosition(device.getName() + "\n" + device.getAddress()) == -1
                            && device.getName() != null && device.getName().contains(getResources().getString(R.string.device_name))) {
                        mNewDevicesAdapter.add(device.getName() + "\n" + device.getAddress());
                        isFinishedDiscovery = false;
                    }
                }
            });


        }

        @Override
        public void receivedFoundDeviceFinished() {
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    mStartDiscovery.setText(R.string.start_discovery);
                    mTitleNewDevices.setText(R.string.title_other_devices);
                    isFinishedDiscovery = true;
                }
            });

        }

    };


    private void bluetoothMode(boolean isEnabled) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                if (mRbBle != null && mRbSpp != null) {
                    if (mAsFingerSDK.deviceType() == DeviceType.ble) {
                        mRbBle.setChecked(true);
                    } else if (mAsFingerSDK.deviceType() == DeviceType.spp) {
                        mRbSpp.setChecked(true);
                    }
                    mRbSpp.setEnabled(isEnabled);
                    mRbBle.setEnabled(isEnabled);
                }

            }
        });

    }

    private void saveBluetoothDevcie() {
        SharedPreferences sp = getSharedPreferences("BluetoothDevice", Context.MODE_PRIVATE);
        SharedPreferences.Editor edit = sp.edit();
        edit.clear();
        if (mAsFingerSDK.connectedDevice() != null)
            edit.putString("address", mAsFingerSDK.connectedDevice().getAddress());
        if (mAsFingerSDK.deviceType() != null)
            edit.putInt("type", mAsFingerSDK.deviceType().getType());
        edit.commit();
    }

}
